<?php
 
class FrmRegAppController{
    function FrmRegAppController(){
        add_action('frm_add_form_option_section', array( &$this, 'registration_options'));
        add_action('wp_ajax_frm_add_usermeta_row', array(&$this, '_usermeta_row'));
        add_filter('frm_setup_new_form_vars', array(&$this, 'setup_new_vars'));
        add_filter('frm_setup_edit_form_vars', array(&$this, 'setup_edit_vars'));
        add_filter('frm_form_options_before_update', array(&$this, 'update_options'), 15, 2);
        
        add_action('frm_entry_form', array( &$this, 'hidden_form_fields' ));
        add_filter('frm_validate_field_entry', array( &$this, 'validate'), 20, 3);
        add_action('frm_after_create_entry', array( &$this, 'create_user'), 30, 2);
        
        add_action('frm_after_update_entry', array( &$this, 'update_user'), 25, 2); 
        
        add_action('show_user_profile', array(&$this, 'show_usermeta'), 200);
        add_action('edit_user_profile', array(&$this, 'show_usermeta'), 200);
        
        add_action('frm_standalone_route', array(&$this, 'standalone_route'), 10, 2);
    }
    
    function registration_options($values){
        global $frm_ajax_url;
        if(isset($values['id']))
            $fields = FrmField::getAll("fi.form_id='$values[id]' and fi.type not in ('divider', 'html', 'break', 'captcha', 'rte')", ' ORDER BY field_order');
        $echo = true;    
        include(FRM_REG_PATH .'/views/registration_options.php');
    }
    
    function _usermeta_row($meta_name=false, $field_id=''){
        if(!$meta_name and isset($_POST['meta_name']))
            $meta_name = $_POST['meta_name'];
        
        if(isset($_POST['form_id']))
            $fields = FrmField::getAll("fi.form_id='$_POST[form_id]' and fi.type not in ('divider', 'html', 'break', 'captcha', 'rte')", ' ORDER BY field_order');
        $echo = false;   
        include(FRM_REG_PATH .'/views/_usermeta_row.php');
        die();
    }
    
    function setup_new_vars($values){
        $defaults = FrmRegAppHelper::get_default_options();
        foreach ($defaults as $opt => $default){
            $values[$opt] = FrmAppHelper::get_param($opt, $default);
            unset($default);
            unset($opt);
        }
        return $values;
    }
    
    function setup_edit_vars($values){
        $defaults = FrmRegAppHelper::get_default_options();
        foreach ($defaults as $opt => $default){
            if (!isset($values[$opt]))
                $values[$opt] = ($_POST and isset($_POST['options'][$opt])) ? $_POST['options'][$opt] : $default;
            unset($default);
            unset($opt);
        }
        
        return $values;
    }
    
    function update_options($options, $values){
        $defaults = FrmRegAppHelper::get_default_options();
        unset($defaults['reg_usermeta']);
        
        foreach($defaults as $opt => $default){
            $options[$opt] = (isset($values['options'][$opt])) ? $values['options'][$opt] : $default;
            unset($default);
            unset($opt);
        }
        
        $options['reg_usermeta'] = array();
        if(isset($values['options']['reg_usermeta'])){
            foreach($values['options']['reg_usermeta']['meta_name'] as $meta_key => $meta_value){
                if(!empty($meta_value) and !empty($values['options']['reg_usermeta']['field_id'][$meta_key]))
                    $options['reg_usermeta'][$meta_value] = $values['options']['reg_usermeta']['field_id'][$meta_key];
            }
        }

        unset($defaults);
        return $options;
    }
    
    function hidden_form_fields($form){
        if(isset($form->options['registration']) and $form->options['registration']){
            if(isset($form->options['reg_username']) and $form->options['reg_username'])
                echo '<input type="hidden" name="frm_register[username]" value="'. $form->options['reg_username'] .'"/>'."\n";
            
            echo '<input type="hidden" name="frm_register[email]" value="'. $form->options['reg_email'] .'"/>'."\n";
        }
    }
    
    
    function validate($errors, $field, $value){
        if(isset($_POST) and isset($_POST['frm_register']) and !empty($value)){
            global $user_ID;
            if($user_ID) return $errors; //don't check if user is logged in because a new user won't be created anyway
            
            if(isset($_POST['frm_register']['username']) and (int)$field->id == $_POST['frm_register']['username']){
                
                //if there is a username field in the form
                $username = $_POST['item_meta'][$_POST['frm_register']['username']];  
                if(FrmRegAppHelper::username_exists($username))
                    $errors['field'.$field->id] = __('This username is already registered.');

            }
            
            if(isset($_POST['frm_register']['email']) and ((int)$field->id == $_POST['frm_register']['email'])){
                if(!function_exists('email_exists'))
                    require_once(ABSPATH . WPINC . '/registration.php');
                
                //check if email has already been used
                if(email_exists($value) )
            		$errors['field'.$field->id] = __('This email address is already registered.' );
            }
        }
        return $errors;
    }
    
    function create_user($entry_id, $form_id){ //TODO: add wp_noonce
        if (isset($_POST) and isset($_POST['frm_register'])){
            global $user_ID;
            if($user_ID and !is_admin()){
                //if user is already logged-in, then update the user
                $this->update_user($entry_id, $form_id);
                return;
            } 
            
            $form = FrmForm::getOne($form_id);
            $form->options = maybe_unserialize($form->options);

            if(!isset($form->options['registration']) or 
                !$form->options['registration'] or 
                !isset($form->options['reg_email']) or 
                !isset($_POST['item_meta'][$form->options['reg_email']]))
                return;
  
            
            $user_meta = $this->_get_usermeta($form->options);
            
            if(!isset($user_meta['user_pass']) or empty($user_meta['user_pass']))
                $user_meta['user_pass'] = wp_generate_password();
            
            if(empty($form->options['reg_username'])){
                //if the username will be generated from the email
                $parts = explode("@", $user_meta['user_email']);
                $user_meta['user_login'] = FrmRegAppHelper::generate_unique_username($parts[0]);
            }else{
                $user_meta['user_login'] = $_POST['item_meta'][$form->options['reg_username']];
            }
            
            $user_meta['role'] = (isset($form->options['reg_role'])) ? $form->options['reg_role'] : 'subscriber';
            
            if(!function_exists('username_exists'))
                require_once(ABSPATH . WPINC . '/registration.php');
            
            $user_id = wp_insert_user($user_meta);
            $user_id = (int)$user_id;
            if($user_id){
                global $frmdb, $wpdb, $frm_entry_meta;
                // set user id field
                $wpdb->update( $frmdb->entries, array('user_id' => $user_id, 'updated_by' => $user_id), array('id' => $entry_id) );
                $user_field = $frmdb->get_var($frmdb->fields, array('type' => 'user_id', 'form_id' => $form_id));
                $frm_entry_meta->delete_entry_meta($entry_id, $user_field);
                $frm_entry_meta->add_entry_meta($entry_id, $user_field, '', $user_id);
                    
                //Update usermeta
                $this->update_usermeta($form->options, $user_id);
                
                // send new user notifications
                wp_new_user_notification($user_id, $user_meta['user_pass']);
                //wp_new_user_notification($user_id, ''); // sending a blank password only sends notification to admin
                //
                
                
                //log user in
                if(!is_user_logged_in() and !isset($_POST['frm_skip_cookie'])){
                ?>
<script type="text/javascript">
jQuery(document).ready(function($){
jQuery.ajax({type:"POST",url:"<?php echo FRM_SCRIPT_URL; ?>",
data:"controller=registration&action=signon&log=<?php echo $user_meta['user_login'] ?>&pwd=<?php echo $user_meta['user_pass'] ?>"
});
});    
</script>
                <?php
                }
            }
            
        }
    }
    
    function signon(){            
        wp_signon();                  
        //wp_set_current_user($user_id, $user_meta['user_login']);
    }
    
    function update_user($entry_id, $form_id){
        if(!(int)$form_id) return;
        
        $form = FrmForm::getOne($form_id);
        $form->options = maybe_unserialize($form->options);
        
        if(!isset($form->options['registration']) or !$form->options['registration'])
            return;
            
        $user_meta = array();
        $user_meta = $this->_get_usermeta($form->options);
        
        global $user_ID;
        $posted_id = isset($_POST['frm_user_id']) ? $_POST['frm_user_id'] : $user_ID;
        if($posted_id){
            if($user_ID and !is_admin() and ($posted_id != $user_ID)) 
                return; //make sure this record is updated by the owner or from the admin
            $user_meta['ID'] = $posted_id;
        }
        
        if(!empty($form->options['reg_username'])){
            $user_meta['user_login'] = $_POST['item_meta'][$form->options['reg_username']];
        }else{
            $user_data = get_userdata($posted_id);
            $user_meta['user_login'] = $user_data->user_login;
        }
           
        if(!function_exists('username_exists'))
            require_once(ABSPATH . WPINC . '/registration.php');
            
        $user_id = wp_insert_user($user_meta);
        
        if($user_id and is_numeric($user_id))
            $this->update_usermeta($form->options, $user_id);
    }
    
    function show_usermeta(){
        global $profileuser, $frmdb;
        
        $meta_keys = array();
        $form_options = $frmdb->get_col($frmdb->forms, array(), 'options');
        foreach($form_options as $opts){
            $opts = maybe_unserialize($opts);
            if(!isset($opts['reg_usermeta']) or empty($opts['reg_usermeta']))
                continue;
            
            foreach($opts['reg_usermeta'] as $meta_key => $field_id){
                if($meta_key != 'user_url')
                    $meta_keys[$field_id] = $meta_key;
            }
        }
        
        if(!empty($meta_keys))
            include(FRM_REG_PATH .'/views/show_usermeta.php');
    }
    
    function update_usermeta($form_options, $user_ID){
        if(isset($form_options['reg_usermeta']) and !empty($form_options['reg_usermeta'])){
            foreach($form_options['reg_usermeta'] as $meta_key => $field_id){
                if(function_exists('update_user_meta'))
                    update_user_meta($user_ID, $meta_key, $_POST['item_meta'][$field_id]);
                else
                    update_usermeta($user_ID, $meta_key, $_POST['item_meta'][$field_id]);
            }
        }
    }
    
    function _get_usermeta($form_options){
        $user_meta = array('user_email' => sanitize_text_field( $_POST['item_meta'][$form_options['reg_email']] ));

        if(is_numeric($form_options['reg_password']) and !empty($_POST['item_meta'][$form_options['reg_password']]))
            $user_meta['user_pass'] = $_POST['item_meta'][$form_options['reg_password']];
           
        foreach(array('first_name', 'last_name') as $user_field){
            if(is_numeric($form_options['reg_'. $user_field]) and !empty($_POST['item_meta'][$form_options['reg_'. $user_field]]))
                $user_meta[$user_field] = $_POST['item_meta'][$form_options['reg_'. $user_field]];
        }
        
        /*
        * 'user_url' - A string containing the user's URL for the user's web site.
        * 'display_name' - A string that will be shown on the site.
        * 'description' - A string containing content about the user.
        */
        
        return $user_meta;
    }
    
    // Routes for standalone / ajax requests
    function standalone_route($controller, $action){
        if($controller != 'registration')
            return;
            
        if($action == 'signon')
            return $this->signon();
    }

}

?>